"""
:Created: 6 December 2014
:Author: Lucas Connors

"""

from django import forms
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from jsonfield.fields import JSONField, JSONFormField, JSONWidget

from .conditions import CompareCondition
from .exceptions import InvalidConditionError
from .lists import CondList

__all__ = ["ConditionsWidget", "ConditionsFormField", "ConditionsField"]


class ConditionsWidget(JSONWidget):

    # TODO: Use template_name and refactor widget to use Django 1.11's new get_context() method
    # when Django 1.8-1.10 support is dropped
    # https://docs.djangoproject.com/en/1.11/ref/forms/widgets/#django.forms.Widget.get_context
    template_name_dj110_to_dj111_compat = "conditions/conditions_widget.html"

    def __init__(self, *args, **kwargs):
        self.condition_definitions = kwargs.pop("condition_definitions", {})
        if "attrs" not in kwargs:
            kwargs["attrs"] = {}
        if "cols" not in kwargs["attrs"]:
            kwargs["attrs"]["cols"] = 50
        super().__init__(*args, **kwargs)

    def render(self, name, value, attrs=None, renderer=None):
        if isinstance(value, CondList):
            value = value.encode()
        textarea = super().render(name, value, attrs)

        condition_groups = []
        for groupname, group in self.condition_definitions.items():
            conditions_in_group = []
            for condstr, condition in group.items():
                conditions_in_group.append(
                    {
                        "condstr": condstr,
                        "key_required": "true" if condition.key_required() else "false",
                        "keys_allowed": condition.keys_allowed,
                        "key_example": condition.key_example(),
                        "operator_required": "true"
                        if issubclass(condition, CompareCondition)
                        else "false",
                        "operators": condition.operators().keys()
                        if issubclass(condition, CompareCondition)
                        else [],
                        "operand_example": condition.operand_example()
                        if issubclass(condition, CompareCondition)
                        else "",
                        "help_text": condition.help_text(),
                        "description": condition.full_description(),
                    }
                )
            conditions_in_group = sorted(
                conditions_in_group, key=lambda x: x["condstr"]
            )

            condition_groups.append(
                {
                    "groupname": groupname,
                    "conditions": conditions_in_group,
                }
            )
        condition_groups = sorted(condition_groups, key=lambda x: x["groupname"])

        context = {
            "textarea": textarea,
            "condition_groups": condition_groups,
        }

        return mark_safe(
            render_to_string(self.template_name_dj110_to_dj111_compat, context)
        )


class ConditionsFormField(JSONFormField):
    def __init__(self, *args, **kwargs):
        self.condition_definitions = kwargs.pop("condition_definitions", {})
        if "widget" not in kwargs:
            kwargs["widget"] = ConditionsWidget(
                condition_definitions=self.condition_definitions
            )
        super().__init__(*args, **kwargs)

    def clean(self, value):
        """ Validate conditions by decoding result """
        cleaned_json = super().clean(value)
        if cleaned_json is None:
            return

        try:
            CondList.decode(cleaned_json, definitions=self.condition_definitions)
        except InvalidConditionError as e:
            raise forms.ValidationError(
                "Invalid conditions JSON: {error}".format(error=str(e))
            )
        else:
            return cleaned_json


class ConditionsField(JSONField):
    """
    ConditionsField stores information on when the "value" of the
    instance should be considered True.
    """

    def __init__(self, *args, **kwargs):
        self.condition_definitions = kwargs.pop("definitions", {})
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs["condition_definitions"] = self.condition_definitions
        return ConditionsFormField(**kwargs)

    def pre_init(self, value, obj):
        value = super().pre_init(value, obj)
        if isinstance(value, dict):
            value = CondList.decode(value, definitions=self.condition_definitions)
        return value

    def dumps_for_display(self, value):
        if isinstance(value, CondList):
            value = value.encode()
        return super().dumps_for_display(value)

    def get_db_prep_value(self, value, connection, prepared=False):
        if isinstance(value, CondList):
            value = value.encode()
        return super().get_db_prep_value(value, connection, prepared)



from .conditions import CompareCondition, Condition
from .exceptions import InvalidConditionError, UndefinedConditionError
from .fields import ConditionsField, ConditionsFormField, ConditionsWidget
from .lists import CondAllList, CondAnyList, CondList, eval_conditions
from .types import conditions_from_module

__all__ = [
    "Condition",
    "CompareCondition",
    "UndefinedConditionError",
    "InvalidConditionError",
    "ConditionsWidget",
    "ConditionsFormField",
    "ConditionsField",
    "CondList",
    "CondAllList",
    "CondAnyList",
    "eval_conditions",
    "conditions_from_module",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['conditions']

package_data = \
{'': ['*'],
 'conditions': ['static/conditions/img/*',
                'static/conditions/js/*',
                'templates/conditions/*']}

install_requires = \
['Django>=2.2,<4.0a0', 'django-jsonfield>=1.1,<2.0']

setup_kwargs = {
    'name': 'django-conditions',
    'version': '0.9.18',
    'description': 'A Django app that allows creation of conditional logic in admin.',
    'long_description': '# django-conditions\n\n![CI](https://github.com/RevolutionTech/django-conditions/actions/workflows/ci.yml/badge.svg)\n[![codecov](https://codecov.io/gh/RevolutionTech/django-conditions/branch/main/graph/badge.svg)](https://codecov.io/gh/RevolutionTech/django-conditions)\n[![Codacy Badge](https://api.codacy.com/project/badge/Grade/8fccc57f17e44c5496a912adc691fc39)](https://www.codacy.com/app/RevolutionTech/django-conditions)\n[![Dependency Status](https://www.versioneye.com/user/projects/56de7e4cdf573d0048dafc52/badge.svg?style=flat)](https://www.versioneye.com/user/projects/56de7e4cdf573d0048dafc52)\n[![Documentation Status](https://readthedocs.org/projects/django-conditions/badge/?version=latest)](http://django-conditions.readthedocs.org/en/latest/)\n\nMove conditional logic that changes often from code into models so that the logic can be easily modified in admin. Some possible use cases:\n- Segment your user base into cohorts with targeted messaging\n- Provide different rewards to users depending on their expected value\n- In a game, define the winning objectives of a mission/quest\n- and many more...\n\n## Installation\n\nFirst install the `django-conditions` package:\n\n    pip install django-conditions\n\nThen add `conditions` to your `INSTALLED_APPS` setting:\n\n```python\n## settings.py\nINSTALLED_APPS = [\n    ...\n    \'conditions\',\n]\n```\n\n## Basic Usage\n\nStart by defining a condition in code:\n\n```python\n## condition_types.py\nfrom conditions import Condition\n\nclass FullName(Condition):\n    # The name that appears in the db and represents your condition\n    condstr = \'FULL_NAME\'\n\n    # Normal conditions define eval_bool, which takes in a user\n    # and returns a boolean\n    def eval_bool(self, user, **kwargs):\n        return bool(user.first_name and user.last_name)\n```\n\nThen add a ConditionsField to your model:\n\n```python\n## models.py\nfrom django.db import models\nfrom conditions import ConditionsField, conditions_from_module\nimport condition_types\n\nclass Campaign(models.Model):\n    text = models.TextField()\n\n    # The ConditionsField requires the definitions of all possible conditions\n    # conditions_from_module can take an imported module and sort this out for you\n    target = ConditionsField(definitions=conditions_from_module(condition_types))\n```\n\nIn the model\'s change form on admin, you can enter JSON to represent when you want your condition to be satisfied.\n\n```javascript\n{\n    "all": ["FULL_NAME"]\n}\n```\n\nNow you can use the logic you created in admin to determine the outcome of an event:\n\n```python\n## views.py\nfrom django.http import HttpResponse\nfrom conditions import eval_conditions\nfrom models import Campaign\n\ndef profile(request):\n    for campaign in Campaign.objects.all():\n        if eval_conditions(campaign, \'target\', request.user):\n            return HttpReponse(campaign.text)\n\n    return HttpResponse("Nothing new to see.")\n```\n\nUse django-conditions in your Django projects to change simple logic without having to re-deploy, and pass on the power to product managers and other non-engineers.\n\n## More Information\n\nFull documentation is available [on Read The Docs](http://django-conditions.readthedocs.org/).\n',
    'author': 'Lucas Connors',
    'author_email': 'lucas@revolutiontech.ca',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/RevolutionTech/django-conditions',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
